/* MicroLens.h ======================================================================*

*Important
===================================================
Raytrix Lenslet Converting Version 0.3

Copyright: ONLY Available for ACADEMIC and MPEG Standardization Purposes
Production: Nagoya University


*Description
===================================================
Given a set of Raw video captured by multi-focused plenoptic camera (i.e. Ratrix camera, https://raytrix.de/) 
and its microlens calibration data, the provided source code can render multi-view video.

This tool is implemented by:

Fujii Laboratory
Department of Information and Communication Engineering
Graduate School of Engineering
Nagoya University


*Term of Use
===================================================
Term of Use: ANY kind of academic publication, and report using this tool MUST refer to the following TWO references.

[1] Shu Fujita, Sho Mikawa, Mehrdad Teratani(Panahpourtehrani), Keita Takahashi, and Toshiaki Fujii 
“Extracting Multi-View Images from Multi-Focused Plenoptic Camera,” 
International Workshop on Advanced Image Technology (IWAIT)/International Forum on Medical Imaging in Asia (IFMIA), 
in SPIE Digital Library, Singapore, January 2019

[2] ] Mehrdad Teratani(Panahpourtehrani), Shu Fujita, Wenzhe Ouyang, Keita Takahashi, Toshiaki Fujii, 
"A 3-D Imaging System Using Multi-Focused Plenoptic Camera and Tensor Display," 
2018 International Conference on 3D Immersion (IC3D2018) in IEEE Xplore Digital Library, Brussels, Belgium, December 2018 

===================================================

* Class for handling raytrix micro lens information
*
* @file		RenderingRxMV > MicroLens.h
* @author	S. Mikawa 
* @date		2018/02/dd
*======================================================================*/

#pragma once
#include <opencv2/opencv.hpp>
#include <iostream>
#include <string>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#define PI 3.14159265 
enum lensType{ TYPE1 = 0, TYPE2 = 1, TYPE3 = 2, TYPE_ALL = 3 };
enum lensPos{ LEFT, MIDDLE, RIGHT };
enum color{ RED, GREEN, BLUE, WHITE, BLACK };

using namespace cv;
using namespace std;
using namespace boost::property_tree;

class MicroLens
{
public:
	MicroLens();
	~MicroLens();

	void initFromRay();
	void initFromXML(Mat rawImage, string filename);
	void showParam();
	void draw(Mat& rawImage, lensType type, int patchSize = 0);
	lensType getRefLensType(int subGridNum, lensPos pos);
	double diameter;
	double radius;
	double rotation;
	Point num;
	Point2d offset[3];
	Point2d center;
	Point2d subGridRefPos[2][3];
};

